==> musicman-0.8.6.tar.gz: musicman/musicman/musicfilenamelayout.cpp <==
/***************************************************************************
                          musicfilenamelayout.cpp  -  description
                             -------------------
    begin                : Fri May 30 2003
    copyright            : (C) 2003 by Avi Alkalay
    email                : avi@unix.sh
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qregexp.h>
#include <kdebug.h>

#include "musicfilenamelayout.h"
#include "musicinfo.h"
#include "musicman.h"

QString MusicFilenameLayout::extensions="mp3|ogg|wav";

MusicFilenameLayout::MusicFilenameLayout(QString _name, QString _tpl, QRegExp _exp) {
	name=_name;
	tpl=_tpl;
	regexp=_exp;
	addExtensions();

	setSubExpPos();
}

MusicFilenameLayout::MusicFilenameLayout(QString _name, QString _tpl, QString _exp, bool minimal) {
	name=_name;
	tpl=_tpl;
	regexp=QRegExp(_exp);
	addExtensions();
	regexp.setMinimal(minimal);
	regexp.setCaseSensitive(false);

	setSubExpPos();
}

void MusicFilenameLayout::addExtensions() {
	regexp.setPattern(regexp.pattern()+"\\.(" + extensions + ")$");
}

MusicFilenameLayout::~MusicFilenameLayout() {}
MusicFilenameLayout::MusicFilenameLayout() {
	setSubExpPos();
}

/** Tests if the filename matches this layout

@param filename the file name to be tested
@returns true if filename matches */
bool MusicFilenameLayout::match(const QString &filename) {
	if (regexp.search(filename)!=-1) return true;
	return false;
}


MusicInfo MusicFilenameLayout::getInfo(const QString &filename) {
	MusicInfo result;
	QString workString=noUnderline(filename); // Remove "_"

	result.extension=getExtension(filename);

	if (regexp.search(workString)!=-1) {
			result.artist=regexp.cap(artistPos);
			result.album=regexp.cap(albumPos);
			result.title=regexp.cap(titlePos);
			result.track=regexp.cap(trackPos).toInt();
	}
	return result;
}







QString MusicFilenameLayout::toggleUnderline(const QString &filename,bool underscore) {
	QString result=filename;

	if (underscore) result.replace(' ','_');
	else result.replace('_',' ');

	return result;
}







FilenameMiner::FilenameMiner() {
	initLayouts();
}








void FilenameMiner::initLayouts() {
	// Matches "Artist Name - Album Name - 05 - Song Name.mp3"
	append(MusicFilenameLayout("Artist Name - Album Name - 05 - Song Name.mp3",
	"%a - %A - %t - %n.%e", "^(.*) - (.*) - (\\d+)(?: - |-| (?!-))(.*)",true));

	// Matches "05 - Song Name.mp3"
	append(MusicFilenameLayout("02 - Song Name.mp3", "%t - %n.%e",
	"^(\\d+)(?: - |-| (?!-))(.*)",false));

	// Matches "Artist Name - Song Name.mp3"
	append(MusicFilenameLayout("Artist Name - Song Name.mp3", "%a - %n.%e",
	"(.*) - (.*)",true));

	// Matches "Song Name.mp3"
	append(MusicFilenameLayout("Song Name.mp3", "%n.%e", "(.*)",false));
}







bool FilenameMiner::match(const QString &filename) {
	FilenameMiner::Iterator it;

	for (it=begin(); it!=end(); it++) {
		if ((*it).match(filename)) return true;
	}
	return false;
}








MusicInfo FilenameMiner::mine(const QString &filename) {
	FilenameMiner::Iterator it;
	MusicInfo info;

	for (it=begin(); it!=end(); it++) {
		info=(*it).getInfo(filename);
		if (!info.isNull()) return info;
	}
	return info;
}












/** Returns the list of layout representations. */
QStringList FilenameMiner::getLayouts() {
	FilenameMiner::Iterator it;
	QStringList result;

	for (it=begin(); it!=end(); it++) {
		result.append((*it).name);
	}
	return result;
}









QString MusicFilenameLayout::getExtension(const QString &filename) {
	QRegExp nameExtension("(.*)\\.(.*)");

	// Separate name from extension
	nameExtension.search(filename);
	return nameExtension.cap(2);
}







/** Normalize a filename to a standard:
- Space to underline
- Uppercase first letter of each word
- Small caps extension */
bool MusicFilenameLayout::normalize(QString &filename,NormalizationParameters &norm) {
	QString name,extension;
	QRegExp nameExtension("(.*)\\.(.*)");
	QStringList words;
	int spaces;

	// Separate name from extension
	nameExtension.search(filename);
	name=nameExtension.cap(1);
	extension=nameExtension.cap(2);



	// Only user wants each word's first letter uppercased
	if (norm.uppercaseFirst) {
		// Remove underlines (_)
		name=noUnderline(name);

		// Separate words
		spaces=name.contains(' ');
		for (int i=0; i<=spaces; i++)
			words.append(name.section(' ',i,i));

		// Work on each word
		// Uppers all occurences of a word's first letter.
		QStringList::Iterator w;
		for (w=words.begin(); w!=words.end(); w++) {
			QString first;
			int firstPos=0;
			QRegExp exp("(?:\\-|\\s|_|^)(\\D)");

			// Put it in lower case first
			*w=(*w).lower();
			while (exp.search(*w,firstPos)!=-1) {
				firstPos=exp.pos(1);
				first=exp.cap(1).upper();
				*w=(*w).replace(firstPos,1,first);
				++firstPos;
			}
		}

		// Rejoin the words
		w=words.begin();
		name=*w;
		for (w++ ;w!=words.end(); w++) {
			name+=' ';
			name+=*w;
		}
	} // if (norm.uppercaseFirst)


	// Work on extension
	switch (norm.extensionType) {
		case NormalizationParameters::extPlain:
			// Lowercase the extension
			extension=extension.lower();
			break;
		case NormalizationParameters::extUpperAll:
			// Uppercase the extension
			extension=extension.upper();
			break;
/*		case NormalizationParameters::extUpperFirst:
			// Lowercase the extension
			extension=extension.lower();
			break;
*/
	}


	// Rejoin everything
	name+='.';
	name+=extension;



	// Reset underlines (_)
	name=toggleUnderline(name,norm.useUnderline);


	if (name.compare(filename)) {
		// If we changed something....
		filename=name;
		return true;
	}
	return false;
}









/** @returns The filename constructed from the template (%a - %n.%e) and the MusicInfo object. */
QString MusicFilenameLayout::getNewFilename(MusicInfo info, NormalizationParameters &np) {
	QString newFilename=tpl;
	QString tmp;

	newFilename.replace(QRegExp("%a"),info.artist);
	newFilename.replace(QRegExp("%n"),info.title);
	newFilename.replace(QRegExp("%A"),info.album);
	newFilename.replace(QRegExp("%t"),tmp.sprintf("%02d",info.track));
	newFilename.replace(QRegExp("%y"),QString::number(info.year));
	newFilename.replace(QRegExp("%b"),QString::number(info.bitrate));
	newFilename.replace(QRegExp("%e"),info.extension);

	if (np.useUnderline) newFilename=yesUnderline(newFilename);

	kdDebug(0) << "New filename: " << newFilename << endl;
  	
	return newFilename;
}





/** Guesses the titlePos, albumPos, artistPos, trackPos and extPos from the template. */
void MusicFilenameLayout::setSubExpPos(void) {
	int posIndex=0,cursor=0;
	QRegExp rx("%.");

	// Simple initialization
	artistPos=titlePos=albumPos=trackPos=yearPos=bitratePos=extensionPos=-1;

	if (tpl.isEmpty()) return;

	while ((cursor=rx.search(tpl,cursor))!=-1) {
		posIndex++;
		cursor+=rx.matchedLength();
		if (rx.cap() == "%a") {
			artistPos=posIndex;
		} else if (rx.cap() == "%n") {
			titlePos=posIndex;
		} else if (rx.cap() == "%A") {
			albumPos=posIndex;
		} else if (rx.cap() == "%t") {
			trackPos=posIndex;
		} else if (rx.cap() == "%y") {
			yearPos=posIndex;
		} else if (rx.cap() == "%b") {
			bitratePos=posIndex;
		} else if (rx.cap() == "%e") {
			extensionPos=posIndex;
		}
	}
}



QString MusicFilenameLayout::yesUnderline(const QString &filename) {
	return toggleUnderline(filename,true);
}

QString MusicFilenameLayout::noUnderline(const QString &filename) {
	return toggleUnderline(filename,false);
}

==> musicman-0.11.tar.gz: musicman/musicman/musicfilenamelayout.cpp <==
/***************************************************************************
                          musicfilenamelayout.cpp  -  description
                             -------------------
    begin                : Fri May 30 2003
    copyright            : (C) 2003 by Avi Alkalay
    email                : avi@unix.sh
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qregexp.h>
#include <kdebug.h>

#include "musicfilenamelayout.h"
#include "musicinfo.h"
#include "musicman.h"

QString MusicFilenameLayout::extensions="mp3|ogg|flac|wav";

MusicFilenameLayout::MusicFilenameLayout(QString _name, QString _tpl, QRegExp _exp) {
	name=_name;
	tpl=_tpl;
	regexp=_exp;
	addExtensions();

	setSubExpPos();
}

MusicFilenameLayout::MusicFilenameLayout(QString _name, QString _tpl, QString _exp, bool minimal) {
	name=_name;
	tpl=_tpl;
	regexp=QRegExp(_exp);
	addExtensions();
	regexp.setMinimal(minimal);
	regexp.setCaseSensitive(false);

	setSubExpPos();
}

void MusicFilenameLayout::addExtensions() {
	regexp.setPattern(regexp.pattern()+"\\.(" + extensions + ")$");
}

MusicFilenameLayout::~MusicFilenameLayout() {}
MusicFilenameLayout::MusicFilenameLayout() {
	setSubExpPos();
}

/** Tests if the filename matches this layout

@param filename the file name to be tested
@returns true if filename matches */
bool MusicFilenameLayout::match(const QString &filename) {
	if (regexp.search(filename)!=-1) return true;
	return false;
}


MusicInfo MusicFilenameLayout::getInfo(const QString &filename) {
	MusicInfo result;
	QString workString=noUnderline(filename); // Remove "_"

	result.extension=getExtension(filename);

	if (regexp.search(workString)!=-1) {
			result.artist=regexp.cap(artistPos);
			result.album=regexp.cap(albumPos);
			result.title=regexp.cap(titlePos);
			result.track=regexp.cap(trackPos).toInt();
	}
	return result;
}







QString MusicFilenameLayout::toggleUnderline(const QString &filename,bool underscore) {
	QString result=filename;

	if (underscore) result.replace(' ','_');
	else result.replace('_',' ');

	return result;
}







FilenameMiner::FilenameMiner() {
	initLayouts();
}








void FilenameMiner::initLayouts() {
	// Matches "Artist Name - Album Name - 05 - Song Name.mp3"
	append(MusicFilenameLayout("Artist Name - Album Name - 05 - Song Name.mp3",
	"%a - %A - %t - %n.%e", "^(.*) - (.*) - (\\d+)(?: - |-| (?!-))(.*)",true));

	// Matches "05 - Song Name.mp3"
	append(MusicFilenameLayout("02 - Song Name.mp3", "%t - %n.%e",
	"^(\\d+)(?: - |-| (?!-))(.*)",false));

	// Matches "Artist Name - Song Name.mp3"
	append(MusicFilenameLayout("Artist Name - Song Name.mp3", "%a - %n.%e",
	"(.*) - (.*)",true));

	// Matches "Song Name.mp3"
	append(MusicFilenameLayout("Song Name.mp3", "%n.%e", "(.*)",false));
}







bool FilenameMiner::match(const QString &filename) {
	FilenameMiner::Iterator it;

	for (it=begin(); it!=end(); it++) {
		if ((*it).match(filename)) return true;
	}
	return false;
}








MusicInfo FilenameMiner::mine(const QString &filename) {
	FilenameMiner::Iterator it;
	MusicInfo info;

	for (it=begin(); it!=end(); it++) {
		info=(*it).getInfo(filename);
		if (!info.isNull()) return info;
	}
	return info;
}












/** Returns the list of layout representations. */
QStringList FilenameMiner::getLayouts() {
	FilenameMiner::Iterator it;
	QStringList result;

	for (it=begin(); it!=end(); it++) {
		result.append((*it).name);
	}
	return result;
}









QString MusicFilenameLayout::getExtension(const QString &filename) {
	QRegExp nameExtension("(.*)\\.(.*)");

	// Separate name from extension
	nameExtension.search(filename);
	return nameExtension.cap(2);
}







/** Normalize a filename to a standard:
- Space to underline
- Uppercase first letter of each word
- Small caps extension */
bool MusicFilenameLayout::normalize(QString &filename,NormalizationParameters &norm) {
	QString name,extension;
	QRegExp nameExtension("(.*)\\.(.*)");
	QStringList words;
	int spaces;

	// Separate name from extension
	nameExtension.search(filename);
	name=nameExtension.cap(1);
	extension=nameExtension.cap(2);


	if (name.isEmpty()) name=filename;  // to handle directories

	
	// Only if user wants each word's first letter uppercased
	if (norm.uppercaseFirst) {
		// Remove underlines (_)
		name=noUnderline(name);

		// Separate words
		spaces=name.contains(' ');
		for (int i=0; i<=spaces; i++)
			words.append(name.section(' ',i,i));

		// Work on each word
		// Uppers all occurences of a word's first letter.
		QStringList::Iterator w;
		for (w=words.begin(); w!=words.end(); w++) {
			QString first;
			int firstPos=0;
			QRegExp exp("(?:\\-|\\s|_|^)(\\D)");

			// Put it in lower case first
			*w=(*w).lower();
			while (exp.search(*w,firstPos)!=-1) {
				firstPos=exp.pos(1);
				first=exp.cap(1).upper();
				*w=(*w).replace(firstPos,1,first);
				++firstPos;
			}
		}

		// Rejoin the words
		w=words.begin();
		name=*w;
		for (w++ ;w!=words.end(); w++) {
			name+=' ';
			name+=*w;
		}
	} // if (norm.uppercaseFirst)


	// Work on extension
	switch (norm.extensionType) {
		case NormalizationParameters::extPlain:
			// Lowercase the extension
			extension=extension.lower();
			break;
		case NormalizationParameters::extUpperAll:
			// Uppercase the extension
			extension=extension.upper();
			break;
		case NormalizationParameters::extUpperFirst:
			// Uppercase first, lowercase the rest
			extension=extension.left(1).upper()+extension.right(extension.length()-1).lower();
			break;
	}


	// Rejoin everything
	if (!extension.isEmpty()) {
		name+='.';
		name+=extension;
	}



	// Reset underlines (_)
	name=toggleUnderline(name,norm.useUnderline);


	if (name.compare(filename)) {
		// If we changed something....
		filename=name;
		return true;
	}
	return false;
}









/** @returns The filename constructed from the template (%a - %n.%e) and the MusicInfo object. */
QString MusicFilenameLayout::getNewFilename(MusicInfo info, NormalizationParameters &np) {
	QString newFilename=tpl;
	QString tmp;
	QString sprintfFormat;

	tmp=info.artist;
	tmp.replace('/',np.slashReplace);
	newFilename.replace(QRegExp("%a"),tmp);
	tmp=info.title;
	tmp.replace('/',np.slashReplace);
	newFilename.replace(QRegExp("%n"),tmp);
	tmp=info.album;
	tmp.replace('/',np.slashReplace);
	newFilename.replace(QRegExp("%A"),tmp);


	sprintfFormat.sprintf("%%0%dd",np.digitsForTrack);
	newFilename.replace(QRegExp("%t"),tmp.sprintf(sprintfFormat,info.track));
	newFilename.replace(QRegExp("%y"),QString::number(info.year));
	newFilename.replace(QRegExp("%b"),QString::number(info.bitrate));
	newFilename.replace(QRegExp("%e"),info.extension);

	// newFilename.replace('/',np.slashReplace);
	if (np.useUnderline) newFilename=yesUnderline(newFilename);

//	kdDebug(0) << "New filename: " << newFilename << endl;
  	
	return newFilename;
}





/** Guesses the titlePos, albumPos, artistPos, trackPos and extPos from the template. */
void MusicFilenameLayout::setSubExpPos(void) {
	int posIndex=0,cursor=0;
	QRegExp rx("%.");

	// Simple initialization
	artistPos=titlePos=albumPos=trackPos=yearPos=bitratePos=extensionPos=-1;

	if (tpl.isEmpty()) return;

	while ((cursor=rx.search(tpl,cursor))!=-1) {
		posIndex++;
		cursor+=rx.matchedLength();
		if (rx.cap() == "%a") {
			artistPos=posIndex;
		} else if (rx.cap() == "%n") {
			titlePos=posIndex;
		} else if (rx.cap() == "%A") {
			albumPos=posIndex;
		} else if (rx.cap() == "%t") {
			trackPos=posIndex;
		} else if (rx.cap() == "%y") {
			yearPos=posIndex;
		} else if (rx.cap() == "%b") {
			bitratePos=posIndex;
		} else if (rx.cap() == "%e") {
			extensionPos=posIndex;
		}
	}
}



QString MusicFilenameLayout::yesUnderline(const QString &filename) {
	return toggleUnderline(filename,true);
}

QString MusicFilenameLayout::noUnderline(const QString &filename) {
	return toggleUnderline(filename,false);
}
